package com.chelvc.framework.security.context;

import java.util.Set;
import java.util.stream.Stream;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.security.crypto.SecurityCipherHandler;
import lombok.NonNull;

/**
 * 应用安全相关操作上下文工具类
 *
 * @author Woody
 * @date 2024/5/24
 */
public final class SecurityContextHolder {
    /**
     * 默认权限分组
     */
    public static final String DEFAULT_PERMISSION_GROUP = "DEFAULT";

    /**
     * 数据安全加解密处理器
     */
    private static volatile SecurityCipherHandler SECURITY_CIPHER_HANDLER;

    private SecurityContextHolder() {
    }

    /**
     * 获取数据安全加解密处理器
     *
     * @return 数据安全加解密处理器实例
     */
    public static SecurityCipherHandler getSecurityCipherHandler() {
        if (SECURITY_CIPHER_HANDLER == null) {
            synchronized (SecurityCipherHandler.class) {
                if (SECURITY_CIPHER_HANDLER == null) {
                    SECURITY_CIPHER_HANDLER = ApplicationContextHolder.getBean(SecurityCipherHandler.class);
                }
            }
        }
        return SECURITY_CIPHER_HANDLER;
    }

    /**
     * 判断当前用户是否拥有任意权限
     *
     * @param permissions 权限标识数组
     * @return true/false
     */
    public static boolean hashAnyPermission(@NonNull String... permissions) {
        if (ObjectUtils.isEmpty(permissions)) {
            return true;
        }
        Session session = SessionContextHolder.getSession(false);
        Set<String> authorities = ObjectUtils.ifNull(session, Session::getAuthorities);
        if (ObjectUtils.notEmpty(authorities)) {
            Stream<String> stream = Stream.of(permissions);
            if (stream.anyMatch(authorities::contains)) {
                return true;
            }
            for (String authority : authorities) {
                Set<?> properties = ApplicationContextHolder.getProperty(authority, Set.class);
                if (ObjectUtils.notEmpty(properties) && stream.anyMatch(properties::contains)) {
                    return true;
                }
            }
        }
        return false;
    }
}
